import os

from django.conf import settings

AUTH_API_TOKEN = os.getenv('AUTH_API_TOKEN') or getattr(
    settings,
    'AUTH_API_TOKEN',
    'CHANGEME'
)

AUTH_API_BASE_URL = os.getenv('AUTH_API_BASE_URL') or getattr(
    settings,
    'AUTH_API_BASE_URL',
    'https://auth.kagiso.io/api/v1'
)



import json

from django.conf import settings
import responses

from ... import http


def get_user_by_email(id, email, status=http.HTTP_200_OK, **kwargs):
    url = 'https://auth.kagiso.io/api/v1/users/{email}/.json'.format(email=email)  # noqa
    data = {
        'id': id,
        'email': email,
        'first_name': kwargs.get('first_name', ''),
        'last_name': kwargs.get('last_name', ''),
        'is_staff': kwargs.get('is_staff', False),
        'is_superuser': kwargs.get('is_superuser', False),
        'confirmation_token': '49:1YkTO2:1VuxvGJre66xqQj6rkEXewmVs08',
        'email_confirmed': None,
        'profile': kwargs.get('profile'),
        'created_via': settings.APP_NAME,
        'created': '2015-04-21T08:18:30.368602Z',
        'modified': '2015-04-21T08:18:30.374410Z'
    }

    responses.add(
        responses.GET,
        url,
        body=json.dumps(data),
        status=status
    )

    return url, data


def post_users(id, email, status=http.HTTP_201_CREATED, **kwargs):
    url = 'https://auth.kagiso.io/api/v1/users/.json'
    data = {
        'id': id,
        'email': email,
        'first_name': kwargs.get('first_name', ''),
        'last_name': kwargs.get('last_name', ''),
        'is_staff': kwargs.get('is_staff', False),
        'is_superuser': kwargs.get('is_superuser', False),
        'confirmation_token': '49:1YkTO2:1VuxvGJre66xqQj6rkEXewmVs08',
        'email_confirmed': None,
        'profile': kwargs.get('profile'),
        'created_via': settings.APP_NAME,
        'created': '2015-04-21T08:18:30.368602Z',
        'modified': '2015-04-21T08:18:30.374410Z'
    }

    responses.add(
        responses.POST,
        url,
        body=json.dumps(data),
        status=status
    )

    return url, data


def put_users(id, email, status=http.HTTP_200_OK, **kwargs):
    url = 'https://auth.kagiso.io/api/v1/users/{id}/.json'.format(id=id)
    data = {
        'id': 1,
        'email': email,
        'first_name': kwargs.get('first_name', ''),
        'last_name': kwargs.get('last_name', ''),
        'is_staff': kwargs.get('is_staff', False),
        'is_superuser': kwargs.get('is_superuser', False),
        'profile': kwargs.get('profile'),
        'created': '2015-04-21T08:18:30.368602Z',
        'created_via': kwargs.get('created_via'),
        'modified': '2015-04-21T08:18:30.374410Z',
        'last_sign_in_via': kwargs.get('last_sign_in_via'),
    }

    responses.add(
        responses.PUT,
        url,
        body=json.dumps(data),
        status=status
    )

    return url, data


def delete_users(id, status=http.HTTP_204_NO_CONTENT):
    url = 'https://auth.kagiso.io/api/v1/users/{id}/.json'.format(id=id)

    responses.add(
        responses.DELETE,
        url,
        status=status
    )

    return url


def post_confirm_email(status=http.HTTP_200_OK):
    url = 'https://auth.kagiso.io/api/v1/confirm_email/.json'  # noqa

    responses.add(
        responses.POST,
        url,
        status=status
    )

    return url


def get_regenerate_confirmation_token(email, status=http.HTTP_200_OK):
    url = 'https://auth.kagiso.io/api/v1/users/{email}/confirmation_token/.json'.format(email=email)  # noqa
    data = {
        'confirmation_token': 'random_token',
    }

    responses.add(
        responses.GET,
        url,
        body=json.dumps(data),
        status=status,
    )

    return url, data


def get_reset_password(email, status=http.HTTP_200_OK):
    url = 'https://auth.kagiso.io/api/v1/reset_password/{email}/.json'.format(email=email)  # noqa
    data = {
        'reset_password_token': 'random_token',
    }

    responses.add(
        responses.GET,
        url,
        body=json.dumps(data),
        status=status,
    )

    return url, data


def post_reset_password(email, status=http.HTTP_200_OK):
    url = 'https://auth.kagiso.io/api/v1/reset_password/{email}/.json'.format(email=email)  # noqa

    responses.add(
        responses.POST,
        url,
        status=status
    )

    return url


def post_sessions(status=http.HTTP_200_OK, **kwargs):
    url = 'https://auth.kagiso.io/api/v1/sessions/.json'
    data = {
        'id': kwargs.get('id', 1),
        'email': kwargs.get('email'),
        'first_name': kwargs.get('first_name', ''),
        'last_name': kwargs.get('last_name', ''),
        'is_staff': kwargs.get('is_staff', False),
        'is_superuser': kwargs.get('is_superuser', False),
        'email_confirmed': None,
        'profile': kwargs.get('profile'),
        'created': '2015-04-21T08:18:30.368602Z',
        'created_via': kwargs.get('created_via'),
        'modified': '2015-04-21T08:18:30.374410Z',
        'last_sign_in_via': kwargs.get('last_sign_in_via'),
    }

    responses.add(
        responses.POST,
        url,
        body=json.dumps(data),
        status=status
    )

    return url, data


def delete_sessions(id, status=http.HTTP_200_OK):
    url = 'https://auth.kagiso.io/api/v1/sessions/{id}/.json'.format(id=id)

    responses.add(
        responses.DELETE,
        url,
        status=status
    )

    return url



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='KagisoUser',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, verbose_name='ID', serialize=False)),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(verbose_name='last login', default=django.utils.timezone.now)),
                ('is_superuser', models.BooleanField(help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status', default=False)),
                ('email', models.EmailField(max_length=250, unique=True)),
                ('is_active', models.BooleanField(default=True)),
                ('date_joined', models.DateTimeField(auto_now_add=True)),
                ('groups', models.ManyToManyField(help_text='The groups this user belongs to. A user will get all permissions granted to each of his/her group.', related_query_name='user', related_name='user_set', blank=True, verbose_name='groups', to='auth.Group')),
                ('user_permissions', models.ManyToManyField(help_text='Specific permissions for this user.', related_query_name='user', related_name='user_set', blank=True, verbose_name='user permissions', to='auth.Permission')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
    ]



class AuthAPIError(Exception):
    pass


class AuthAPINetworkError(AuthAPIError):
    pass


class AuthAPITimeout(AuthAPIError):
    pass


class AuthAPIUnexpectedStatusCode(AuthAPIError):

    def __init__(self, status_code, json):
        message = 'Status: {0}.\nJson: {1}'.format(status_code, json)
        self.status_code = status_code
        self.json = json
        super().__init__(message)


class EmailNotConfirmedError(AuthAPIError):
    pass



from authomatic import Authomatic
from authomatic.adapters import DjangoAdapter
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from django.core.mail import EmailMessage
from django.core.urlresolvers import reverse
from django.db.utils import IntegrityError
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.utils.safestring import mark_safe
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt

from . import forms
from .exceptions import EmailNotConfirmedError
from .models import KagisoUser
from .utils import get_setting


@never_cache
@csrf_exempt
def sign_up(request):
    confirm_message = (
        'You will receive an email with confirmation instructions shortly. '
        'This link will expire within 24 hours.'
    )
    error_message = 'You already have an account.'
    next = request.GET.get('next', '/')

    if request.user.is_authenticated():
        return HttpResponseRedirect(next)

    oauth_data = request.session.get('oauth_data')

    if request.method == 'POST':
        form = forms.SignUpForm.create(
            post_data=request.POST,
            oauth_data=oauth_data
        )

        if form.is_valid():
            try:
                user = form.save(
                    app_name=get_setting(settings.APP_NAME, request)
                )
            except IntegrityError:
                messages.error(request, error_message)
                return HttpResponseRedirect(reverse('sign_in'))

            # Social sign ins provide emails that have already been confirmed
            # via FB, Google etc...
            if not oauth_data:
                _send_confirmation_email(user, request)
                messages.success(request, confirm_message)
                return HttpResponseRedirect(next)

            _social_login(request, user.email, oauth_data['provider'])
            return HttpResponseRedirect(next)
    else:
        form = forms.SignUpForm.create(oauth_data=oauth_data)

    return render(
        request,
        'kagiso_auth/sign_up.html',
        {
            'form': form,
            'next': next,
        }
    )


def _send_confirmation_email(user, request):
    msg = EmailMessage()
    msg.template_name = get_setting(settings.SIGN_UP_EMAIL_TEMPLATE, request)
    msg.from_email = 'noreply@kagisomedia.co.za'
    msg.subject = 'Confirm Your Account'
    msg.to = [user.email]
    msg.global_merge_vars = {
        'link': request.build_absolute_uri(reverse('confirm_account')),
        'token': user.confirmation_token,
        'user_id': user.id,
        'first_name': user.first_name
    }
    msg.use_template_subject = True
    msg.use_template_from = True
    msg.send()


@never_cache
def confirm_account(request):
    confirm_message = 'We have confirmed your details, please sign in below'

    user_id = request.GET.get('user_id')
    token = request.GET.get('token')

    user = get_object_or_404(KagisoUser, id=user_id)
    user.confirm_email(token)

    messages.success(request, confirm_message)
    return HttpResponseRedirect(reverse('sign_in'))


@never_cache
@csrf_exempt
def sign_in(request):
    next = request.GET.get('next', '/')

    if request.user.is_authenticated():
        return HttpResponseRedirect(next)

    if request.method == 'POST':
        form = forms.SignInForm(request.POST)
        if form.is_valid():
            email = form.cleaned_data['email']
            password = form.cleaned_data['password']
            try:
                user = authenticate(
                    email=email,
                    password=password,
                )

                if user:
                    user.last_sign_in_via = get_setting(
                        settings.APP_NAME,
                        request
                    )
                    user.save()
                    response = HttpResponseRedirect(next)
                    response.set_cookie('signed_in',
                                        value='true',
                                        max_age=2544)
                    login(request, user)
                    return response
                else:
                    messages.error(request, 'Incorrect email or password')
            except EmailNotConfirmedError:
                resend_message = (
                    'Please first confirm your email address. '
                    '<a href="/resend_confirmation?email={email}">'
                    'Resend confirmation email</a>'
                ).format(email=email)
                messages.error(
                    request,
                    mark_safe(resend_message)
                )
    else:
        form = forms.SignInForm()

    return render(
        request,
        'kagiso_auth/sign_in.html',
        {
            'form': form,
            'next': next,
        }
    )


@never_cache
def oauth(request, provider):
    response = HttpResponse()
    authomatic = Authomatic(
        get_setting(settings.AUTHOMATIC_CONFIG, request),
        get_setting(settings.SECRET_KEY, request)
    )
    result = authomatic.login(DjangoAdapter(request, response), provider)

    if result:
        if result.error:
            messages.error(
                request,
                'There was an error signing you in. Please try again'
            )
            return HttpResponseRedirect(reverse('sign_in'))

        if result.user:
            # Then user is being redirected back from provider with their data
            #
            # OAuth 2.0 and OAuth 1.0a provide only limited user data on login,
            # We need to update the user to get more info.
            if not (result.user.name and result.user.id):
                result.user.update()

            email = result.user.data.get('email')
            provider = result.provider.name
            user = KagisoUser.get_user_from_auth_db(email)

            if user:
                _social_login(request, user.email, provider)
                return HttpResponseRedirect('/')
            else:
                gender = result.user.gender

                if gender:
                    # Form constants (region, gender) are in uppercase
                    gender = gender.upper()

                data = {
                    'provider': provider,
                    'email': result.user.email,
                    'first_name': result.user.first_name,
                    'last_name': result.user.last_name,
                    'gender': gender,
                    'birth_date': result.user.birth_date,
                }

                request.session['oauth_data'] = data
                return HttpResponseRedirect(reverse('sign_up'))

    # If result.user is None then user will be redirected to provider
    # to authenticate themselves, prior to being redirected back to this view.
    return response


def _social_login(request, email, provider):
    user = authenticate(
        email=email,
        strategy=provider,
    )
    login(request, user)


@never_cache
@login_required
def sign_out(request):
    request.user.record_sign_out()
    response = HttpResponseRedirect('/')
    logout(request)
    return response


@never_cache
@csrf_exempt
def forgot_password(request):
    reset_message = 'You will receive an email with reset instructions shortly'
    not_found_message = 'We could not find a user for that email address'

    if request.method == 'POST':
        form = forms.ForgotPasswordForm(request.POST)

        if form.is_valid():
            email = form.cleaned_data['email']
            user = KagisoUser.get_user_from_auth_db(email)

            if user:
                msg = EmailMessage()
                msg.template_name = get_setting(
                    settings.PASSWORD_RESET_EMAIL_TEMPLATE,
                    request
                )
                msg.from_email = 'noreply@kagisomedia.co.za'
                msg.subject = 'Password Reset'
                msg.to = [user.email]
                msg.global_merge_vars = {
                    'link': request.build_absolute_uri
                    (reverse('reset_password')),
                    'token': user.generate_reset_password_token(),
                    'user_id': user.id
                }
                msg.use_template_subject = True
                msg.use_template_from = True
                msg.send()

                messages.success(request, reset_message)
                return HttpResponseRedirect('/')
            else:
                messages.error(request, not_found_message)
                return HttpResponseRedirect(reverse('forgot_password'))
    else:
        form = forms.ForgotPasswordForm()

    return render(
        request,
        'kagiso_auth/forgot_password.html',
        {'form': form}
    )


@never_cache
@csrf_exempt
def reset_password(request):
    reset_message = 'Your password has been reset'

    if request.method == 'POST':
        form = forms.ResetPasswordForm(request.POST)

        if form.is_valid():
            # user_id is just for RESTFUL routing...
            # Token includes user_id and is validated server-side for tampering
            user_id = form.cleaned_data['user_id']
            token = form.cleaned_data['token']
            password = form.cleaned_data['password']

            user = KagisoUser.objects.filter(id=user_id).first()
            if user:
                user.reset_password(password, token)
                messages.success(request, reset_message)

            return HttpResponseRedirect('/')
    else:
        form = forms.ResetPasswordForm(
            initial={
                'user_id': request.GET.get('user_id'),
                'token': request.GET.get('token', ''),
            }
        )

    return render(
        request,
        'kagiso_auth/reset_password.html',
        {'form': form}
    )


@never_cache
@csrf_exempt
def resend_confirmation(request):
    not_found_message = 'We could not find a user for that email address'
    user = KagisoUser.objects.filter(email=request.GET['email']).first()

    if not user:
        messages.error(request, not_found_message)
        return HttpResponseRedirect('/')

    _send_confirmation_email(user, request)

    confirm_message = (
        'You will receive an email with confirmation instructions shortly. '
        'This link will expire within 24 hours.'
    )
    messages.success(request, confirm_message)

    return HttpResponseRedirect(reverse('sign_in'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('kagiso_auth', '0002_auto_20150421_0835'),
    ]

    operations = [
        migrations.AddField(
            model_name='kagisouser',
            name='profile',
            field=jsonfield.fields.JSONField(null=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kagiso_auth', '0005_kagisouser_email_confirmed'),
    ]

    operations = [
        migrations.AddField(
            model_name='kagisouser',
            name='first_name',
            field=models.CharField(null=True, max_length=100, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='kagisouser',
            name='is_staff',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='kagisouser',
            name='last_name',
            field=models.CharField(null=True, max_length=100, blank=True),
            preserve_default=True,
        ),
    ]



from datetime import date
from unittest.mock import MagicMock, patch

from django.core import mail
from django.db.utils import IntegrityError
from django.test import RequestFactory, TestCase

from ... import views
from ...exceptions import EmailNotConfirmedError
from ...models import KagisoUser


class SignUpTest(TestCase):

    def test_sign_up_get(self):
        response = self.client.get('/sign_up/')

        assert response.status_code == 200
        assert b'<h1>Register for an account</h1>' in response.content

    @patch('kagiso_auth.forms.KagisoUser', autospec=True)
    def test_sign_up_post(self, MockKagisoUser):  # noqa
        mock_user = MockKagisoUser.return_value
        mock_user.id = 1
        mock_user.save.return_value = mock_user

        data = {
            'email': 'bogus@email.com',
            'first_name': 'Fred',
            'last_name': 'Smith',
            'password': 'mypassword',
            'confirm_password': 'mypassword',
            'mobile': '0821111111',
            'gender': 'MALE',
            'region': 'EASTERN_CAPE',
            'birth_date': date(1980, 1, 31),
            'alerts': ['EMAIL', 'SMS'],
        }

        response = self.client.post('/sign_up/', data, follow=True)

        message = list(response.context['messages'])[0].message

        assert response.status_code == 200
        assert mock_user.save.called

        assert mock_user.email == data['email']
        assert mock_user.first_name == data['first_name']
        assert mock_user.last_name == data['last_name']
        assert mock_user.profile['mobile'] == data['mobile']
        assert mock_user.profile['gender'] == data['gender']
        assert mock_user.profile['region'] == data['region']
        assert mock_user.profile['birth_date'] == str(data['birth_date'])
        assert mock_user.profile['alerts'] == data['alerts']

        assert (
            'You will receive an email with confirmation instructions shortly. '  # noqa
            'This link will expire within 24 hours.'
        ) == message

        assert len(mail.outbox) == 1
        assert mail.outbox[0].to[0] == mock_user.email
        assert mail.outbox[0].subject == 'Confirm Your Account'

    @patch('kagiso_auth.forms.KagisoUser', autospec=True)
    def test_sign_up_user_already_exists_redirects_to_sign_in(self, MockKagisoUser):  # noqa
        mock_user = MockKagisoUser.return_value
        mock_user.save.side_effect = IntegrityError()

        data = {
            'email': 'bogus@email.com',
            'first_name': 'Fred',
            'last_name': 'Smith',
            'password': 'mypassword',
            'confirm_password': 'mypassword',
            'mobile': '0821111111',
            'gender': 'MALE',
            'region': 'EASTERN_CAPE',
            'birth_date': date(1980, 1, 31),
            'alerts': ['EMAIL', 'SMS'],
        }

        response = self.client.post('/sign_up/', data, follow=True)

        message = list(response.context['messages'])[0].message

        assert message == 'You already have an account.'
        self.assertRedirects(response, '/sign_in/')

    @patch('kagiso_auth.views.get_object_or_404', autospec=True)
    def test_confirm_account(self, mock_get_object_or_404):
        mock_instance = mock_get_object_or_404.return_value

        response = self.client.get(
            '/confirm_account/?user_id=1&token=my_token',
            follow=True
        )

        message = list(response.context['messages'])[0].message

        assert response.status_code == 200
        assert 'We have confirmed your details, please sign in below' == message  # noqa
        mock_instance.confirm_email.assert_called_with('my_token')
        assert b'<h1>Sign In</h1>' in response.content

    @patch('kagiso_auth.forms.KagisoUser', autospec=True)
    def test_redirects_to_next_url(self, MockKagisoUser):  # noqa
        mock_user = MockKagisoUser.return_value
        mock_user.id = 1
        mock_user.save.return_value = mock_user

        data = {
            'email': 'bogus@email.com',
            'first_name': 'Fred',
            'last_name': 'Smith',
            'password': 'mypassword',
            'confirm_password': 'mypassword',
            'mobile': '0821111111',
            'gender': 'MALE',
            'region': 'EASTERN_CAPE',
            'birth_date': date(1980, 1, 31),
            'alerts': ['EMAIL', 'SMS'],
        }

        url = '/sign_up/?next=/some-url/'
        response = self.client.post(url, data, follow=True)

        assert response.request['PATH_INFO'] == '/some-url/'


class SignInTest(TestCase):

    def test_sign_in_get(self):
        response = self.client.get('/sign_in/')

        assert response.status_code == 200
        assert b'<h1>Sign In</h1>' in response.content

    @patch('kagiso_auth.views.authenticate', autospec=True)
    def test_sign_in_invalid_credentials(self, mock_authenticate):
        data = {'email': 'bogus@email.com', 'password': 'bogus'}
        mock_authenticate.return_value = None

        response = self.client.post('/sign_in/', data, follow=True)
        message = list(response.context['messages'])[0].message

        assert mock_authenticate.called
        assert message == 'Incorrect email or password'

    @patch('kagiso_auth.views.authenticate', autospec=True)
    def test_sign_in_unconfirmed_email(self, mock_authenticate):
        data = {'email': 'bogus@email.com', 'password': 'bogus'}
        mock_authenticate.side_effect = EmailNotConfirmedError

        response = self.client.post('/sign_in/', data, follow=True)
        message = list(response.context['messages'])[0].message

        assert mock_authenticate.called
        assert message == 'Please first confirm your email address. ' \
            '<a href="/resend_confirmation?email={email}">' \
            'Resend confirmation email</a>'.format(email=data['email'])

    @patch('kagiso_auth.views.KagisoUser', autospec=True)
    def test_resend_confirmation(self, MockKagisoUser):  # noqa
        # ----- Arrange -----
        email = 'mock@user.com'
        user = KagisoUser(email=email)

        mock_filter = MagicMock()
        mock_filter.first.return_value = user
        MockKagisoUser.objects.filter.return_value = mock_filter

        data = {'email': email}

        # ----- Act -----
        response = self.client.get('/resend_confirmation/', data, follow=True)

        # ----- Assert -----
        message = list(response.context['messages'])[0].message

        assert response.status_code == 200
        assert (
            'You will receive an email with confirmation instructions shortly. '  # noqa
            'This link will expire within 24 hours.'
        ) == message

        assert len(mail.outbox) == 1
        assert mail.outbox[0].to[0] == email
        assert mail.outbox[0].subject == 'Confirm Your Account'

    @patch('kagiso_auth.views.login', autospec=True)
    @patch('kagiso_auth.views.authenticate', autospec=True)
    def test_sign_in_valid_credentials(self, mock_authenticate, mock_login):
        mock_user = MagicMock()
        data = {'email': 'test@email.com', 'password': 'secret'}
        mock_authenticate.return_value = mock_user

        response = self.client.post('/sign_in/', data, follow=True)

        assert response.status_code == 200
        assert mock_authenticate.called
        assert mock_login.called
        assert mock_user.is_authenticated()

    @patch('kagiso_auth.views.login', autospec=True)
    @patch('kagiso_auth.views.authenticate', autospec=True)
    def test_redirects_to_next_url(self, mock_authenticate, mock_login):
        mock_user = MagicMock()
        data = {'email': 'test@email.com', 'password': 'secret'}
        mock_authenticate.return_value = mock_user

        url = '/sign_in/?next=/some-url/'
        response = self.client.post(url, data, follow=True)

        assert response.request['PATH_INFO'] == '/some-url/'
        assert mock_authenticate.called
        assert mock_login.called
        assert mock_user.is_authenticated()


class OauthTest(TestCase):

    @patch('kagiso_auth.views.KagisoUser', autospec=True)
    @patch('kagiso_auth.views.Authomatic', autospec=True)
    def test_new_user_redirects_to_sign_up_page(  # noqa
        self,
        MockAuthomatic,
        MockKagisoUser):

        oauth_data = {
            'email': 'test@email.com',
            'first_name': 'Fred',
            'last_name': 'Smith',
            'birth_date': str(date(1980, 1, 31)),
            'gender': 'male',
        }
        mock_result = MagicMock()
        mock_result.provider.name = 'facebook'
        mock_result.error = None
        mock_result.user.data = oauth_data

        mock_result.user.email = oauth_data['email']
        mock_result.user.first_name = oauth_data['first_name']
        mock_result.user.last_name = oauth_data['last_name']
        mock_result.user.birth_date = oauth_data['birth_date']
        mock_result.user.gender = oauth_data['gender']

        mock_authomatic = MagicMock()
        mock_authomatic.login.return_value = mock_result
        MockAuthomatic.return_value = mock_authomatic

        MockKagisoUser.get_user_from_auth_db.return_value = None

        response = self.client.get('/oauth/facebook/', follow=True)

        # Social sign up should prefill sign up form
        assert oauth_data['email'] in str(response.content)
        assert oauth_data['first_name'] in str(response.content)
        assert oauth_data['last_name'] in str(response.content)
        assert oauth_data['gender'] in str(response.content)

        assert mock_authomatic.login.called

    @patch('kagiso_auth.views.authenticate', autospec=True)
    @patch('kagiso_auth.views.login', autospec=True)
    @patch('kagiso_auth.views.KagisoUser', autospec=True)
    @patch('kagiso_auth.views.Authomatic', autospec=True)
    def test_existing_user_gets_signed_in(  # noqa
            self,
            MockAuthomatic,
            MockKagisoUser,
            mock_login,
            mock_authenticate):

        user = KagisoUser(email='test@email.com')
        user.save = MagicMock()

        mock_result = MagicMock()
        mock_result.error = None
        mock_result.provider.name = 'facebook'
        mock_authomatic = MagicMock()
        mock_authomatic.login.return_value = mock_result
        MockAuthomatic.return_value = mock_authomatic
        mock_authenticate.return_value = user

        response = self.client.get('/oauth/facebook/', follow=True)

        self.assertRedirects(response, '/')
        assert mock_authomatic.login.called
        assert mock_login.called
        assert mock_authenticate.called

    @patch('kagiso_auth.views.authenticate', autospec=True)
    @patch('kagiso_auth.views.login', autospec=True)
    @patch('kagiso_auth.views.KagisoUser', autospec=True)
    @patch('kagiso_auth.views.Authomatic', autospec=True)
    def test_oauth_error_redirects_to_sign_in_page(  # noqa
            self,
            MockAuthomatic,
            MockKagisoUser,
            mock_login,
            mock_authenticate):

        user = KagisoUser(email='test@email.com')
        user.save = MagicMock()

        mock_result = MagicMock()
        mock_result.error = True

        response = self.client.get('/oauth/facebook/', follow=True)

        self.assertRedirects(response, '/sign_in/')


class SignOutTest(TestCase):

    @patch('kagiso_auth.views.logout', autospec=True)
    def test_sign_out(self, mock_logout):
        site = MagicMock()
        site.hostname = 'jacarandafm.com'
        user = KagisoUser()
        user.record_sign_out = MagicMock()

        request_factory = RequestFactory()
        request = request_factory.get('/sign_out/')
        request.site = site
        request.user = user

        response = views.sign_out(request)

        assert response.status_code == 302
        assert user.record_sign_out.called
        assert mock_logout.called


class ForgotPasswordTest(TestCase):

    def test_forgot_password_get(self):
        response = self.client.get('/forgot_password/')

        assert response.status_code == 200
        assert b'<h1>Forgot Password</h1>' in response.content

    @patch('kagiso_auth.views.KagisoUser', autospec=True)
    def test_forgot_password_post_user_not_found(self, MockKagisoUser):  # noqa
        MockKagisoUser.get_user_from_auth_db.return_value = None
        data = {'email': 'no@user.com'}

        response = self.client.post('/forgot_password/', data, follow=True)
        message = list(response.context['messages'])[0].message

        assert response.status_code == 200
        assert 'We could not find a user for that email address' == message

    @patch('kagiso_auth.views.KagisoUser', autospec=True)
    def test_forgot_password_sends_reset_email(self, MockKagisoUser):  # noqa
        # ----- Arrange -----
        email = 'mock@user.com'

        user = KagisoUser(email=email)
        user.generate_reset_password_token = MagicMock(return_value='token')
        MockKagisoUser.get_user_from_auth_db.return_value = user

        data = {'email': email}

        # ----- Act -----
        response = self.client.post('/forgot_password/', data, follow=True)

        # ----- Assert -----
        message = list(response.context['messages'])[0].message

        assert response.status_code == 200
        assert 'You will receive an email with reset instructions shortly' == message  # noqa

        assert len(mail.outbox) == 1
        assert mail.outbox[0].to[0] == email
        assert mail.outbox[0].subject == 'Password Reset'


class ResetPasswordTest(TestCase):

    def test_reset_password_get(self):
        response = self.client.get('/reset_password/')

        assert response.status_code == 200
        assert b'<h1>Reset Password</h1>' in response.content

    @patch('kagiso_auth.views.KagisoUser', autospec=True)
    def test_reset_password_post(self, MockKagisoUser):  # noqa
        email = 'mock@user.com'
        user = KagisoUser(email=email)
        user.reset_password = MagicMock()

        mock_filter = MagicMock()
        mock_filter.first.return_value = user
        MockKagisoUser.objects.filter.return_value = mock_filter

        data = {
            'email': email,
            'token': 'my_token',
            'password': 'password',
            'confirm_password': 'password'
        }

        response = self.client.post('/reset_password/', data, follow=True)
        message = list(response.context['messages'])[0].message

        assert response.status_code == 200
        assert 'Your password has been reset' == message
        user.reset_password.assert_called_with(data['password'], data['token'])



from django.contrib.auth.backends import ModelBackend

from . import http
from .auth_api_client import AuthApiClient
from .exceptions import AuthAPIUnexpectedStatusCode, EmailNotConfirmedError
from .models import KagisoUser


class KagisoBackend(ModelBackend):

    # Django calls our backend with username='xyz', password='abc'
    # e.g. credentials = {'username': 'Fred', 'password': 'open'}
    # authenticate(**credentials), even though we set USERNAME_FIELD to
    # 'email' in models.py.
    #
    # Django AllAuth does this:
    #  credentials = {'email': 'test@kagiso.io, 'password': 'open'}
    def authenticate(self, email=None, username=None, password=None, **kwargs):
        email = username if not email else email

        payload = {
            'email': email,
        }

        # Social signins don't have passwords
        if password:
            payload['password'] = password

        # Support social sign_ins
        strategy = kwargs.get('strategy')
        if strategy:
            payload['strategy'] = strategy

        auth_api_client = AuthApiClient()
        status, data = auth_api_client.call('sessions', 'POST', payload)

        if status == http.HTTP_200_OK:
            user = KagisoUser.sync_user_data_locally(data)
        elif status == http.HTTP_404_NOT_FOUND:
            return None
        elif status == http.HTTP_422_UNPROCESSABLE_ENTITY:
            raise EmailNotConfirmedError()
        else:
            raise AuthAPIUnexpectedStatusCode(status, data)

        return user



from setuptools import find_packages, setup


setup(
    name='kagiso_django_auth',
    version='5.6.0',
    author='Kagiso Media',
    author_email='development@kagiso.io',
    description='Kagiso Django AuthBackend',
    url='https://github.com/Kagiso-Future-Media/django_auth',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
            'jsonfield==1.0.3',
            'requests==2.8.1',
            'authomatic==0.1.0.post1'
    ],
)



from django.conf import settings
from django.conf.urls import patterns, url
from django.shortcuts import render

from kagiso_auth import views

urlpatterns = patterns(
    '',
    url(r'^sign_up/', views.sign_up, name='sign_up'),
    url(r'^confirm_account/', views.confirm_account, name='confirm_account'),
    url(r'^sign_in/', views.sign_in, name='sign_in'),
    url(r'^sign_out/', views.sign_out, name='sign_out'),
    url(r'^oauth/(?P<provider>\w+)/', views.oauth, name='oauth'),
    url(r'^forgot_password/', views.forgot_password, name='forgot_password'),
    url(r'^reset_password/', views.reset_password, name='reset_password'),
    url(
        r'^resend_confirmation/',
        views.resend_confirmation,
        name='resend_confirmation'
    ),
)

if getattr(settings, 'UNIT_TEST_SETTINGS', False):
    # The host app will have the real home page
    def mock_home(request):
        return render(request, 'kagiso_auth/blank.html')

    urlpatterns.append(url(r'^$', mock_home))
